@@ -26,6 +26,7 @@ class ZMQFrameDecoder(object):
 
     def __init__(self):
         self.version = self.encoder = self.datasize = self.audiolevel_left = self.audiolevel_right = None
+        self._length_valid = False
 
     def load_frame(self, frame):
         frame_header = "<HHIhh"
@@ -33,22 +34,23 @@ class ZMQFrameDecoder(object):
         headersize = struct.calcsize(frame_header)
         if len(frame) > headersize:
             # > instead of >= because we want to ensure there's also payload
-            # TODO check that length of payload matches
 
             unpacked = struct.unpack(frame_header, frame[:headersize])
             self.version, self.encoder, self.datasize, self.audiolevel_left, self.audiolevel_right = unpacked
+            self._length_valid = (len(frame) - headersize) == self.datasize
         else:
             self.version = self.encoder = self.datasize = self.audiolevel_left = self.audiolevel_right = None
+            self._length_valid = False
 
-    def version_is_valid(self):
+    def is_valid(self):
         # See ODR-DabMux' src/input/Zmq.cpp for reference
-        return self.version == 1
+        return self._length_valid and self.version == 1
 
     def get_audio_levels(self):
         """
         Returns a tuple with left and right audio levels, or (None, None) if frame is not valid
         """
-        if not self.version_is_valid():
+        if not self.is_valid():
             return (None, None)
         else:
             int16_max = 0x7FFF
